# Godot Version Manager

Tool to manage Godot versions. Made with Haxe. Mostly tested on Linux.

## Installation (From Source)

Only tested with Haxe 4.3+

- Clone this repo
- Create a folder in the repo named `versions`. You can add godot binaries to this folder
- Run `./build.sh`
  - Windows users can use `haxe build.hxml`.
- Make an alias pointing to the `godot` binary that was created

## Usage

If you haven't added any Godot versions yet, you can do so in 2 ways:

- Add binary files to the `versions` folder in the repo
- Use `godot download <version>`

Once you have at least one version of Godot in the `versions` folder, you can use `godot switch` to select which version you want to use. Running just `godot switch` will change the global version, and using the `-local` flag will change the version used in the current folder.

To run Godot, simply run `godot`. You can pass whatever arguments you want to it, like `godot -e` to run the editor. If for whatever reason you need to run Godot with `switch` or `download` as the first argument, you can use `godot run <arguments>` and it will run Godot as if you ran `godot <arguments>`.

## Advanced Usage

You can force godot-manager to download binaries for a different platform by setting the `GODOT_MANAGER_PLATFORM` environment variable. Valid values are `android.apk` (3.x, 4.x), `linux.64`, `linux.32`, `macos.universal`, `windows.64`, `windows.32`, `web` (3.x, 4.x), `linux_server.headless.64` (3.x only), and `linux_server.64` (1.x, 2.x, 3.x). If not set, or set to an invalid value, godot-manager will use `linux.64` on Linux and BSD, `macos.universal` on MacOS, and `windows.64` on Windows.

You can select which mirror godot-manager downloads godot binaries from using the `GODOT_MANAGER_DOWNLOAD_MIRROR` environment variable. Valid values are `tuxfamily` for `https://downloads.tuxfamily.org/godotengine/`, `github` for `https://github.com/godotengine/godot/releases`, and `github_builds` for `https://github.com/godotengine/godot-builds/releases`. If not set, or set to an invalid value, godot-manager will use `github` for stable builds and `github_builds` for alphas, betas, dev builds, and release candidates.

godot-manager can find a version to use from the following sources, and will use the first one it finds (that exists) in the list below.

- `GODOT_EXECUTABLE` environment variable, with the value set to the full path to the Godot executable to run.
- `GODOT_VERSION` environment variables, with the value set to the name of a file in the `versions` folder, or set to the short version string (eg. `4.1-beta3` or `4.0`)
- `.godot_version` in the current folder, with the value set to the name of a file in the `versions` folder, or set to a short version string (eg. `4.1-beta3` or `4.0`)
- `.godot_version` in the godot-manager folder, with the value set to the name of a file in the `versions` folder, or set to a short version string (eg. `4.1-beta3` or `4.0`)

If no version is found, it will tell you that no version was found, and quit.
