package interfaces;

import ANSI;

/**
   An abstract fullscreen interface
**/
abstract class AbstractInterface {
   var visible:Bool = false;
   var focused:Bool = false;

   public function show():Void {
      ANSI.alternateScreen = true;
      ANSI.reportMouseClicks = true;
      // ANSI.alternateScrollMode = true;
      visible = true;
      focused = true;

      while (visible) {
         print();
         handleInput();
      }
   }

   public function hide():Void {
      ANSI.alternateScreen = false;
      ANSI.reportMouseClicks = false;
      // ANSI.alternateScrollMode = false;
      visible = false;
      focused = false;
   }

   abstract function print():Void;

   function handleInput():Void {
      var char = Sys.getChar(false);

      switch char {
         case 27: // escape sequence start
            var csi = Sys.getChar(false);

            if (csi != '['.code)
               return; // if its not a [ then ignore it

            var action = Sys.getChar(false);

            switch (action : ControlCharacter) {
               case ARROW_UP:
                  arrowUpKey();
               case ARROW_DOWN:
                  arrowDownKey();
               case ARROW_LEFT:
                  arrowLeftKey();
               case ARROW_RIGHT:
                  arrowRightKey();
               case PAGE_UP:
                  Sys.getChar(false);
                  pageUpKey();
               case PAGE_DOWN:
                  Sys.getChar(false);
                  pageDownKey();
               case KEY_HOME:
                  homeKey();
               case KEY_END:
                  endKey();
               case MOUSE_CLICK:
                  var button = Sys.getChar(false) - 32;
                  var x = Sys.getChar(false) - 32;
                  var y = Sys.getChar(false) - 32;
                  click(button, x, y);
               default:
                  // nothing!
            }
         case 13: // enter
            enterKey();
         case 3: // ctrl + c
            hide();
         default:
            // nothing!
      }
   }

   function arrowUpKey():Void {}

   function arrowDownKey():Void {}

   function arrowLeftKey():Void {}

   function arrowRightKey():Void {}

   function pageUpKey():Void {}

   function pageDownKey():Void {}

   function homeKey():Void {}

   function endKey():Void {}

   function enterKey():Void {}

   function click(mouseButton:Int, x:Int, y:Int):Void {}
}
