package interfaces;

import ANSI;

/**
   An interface that presents a list of items.
**/
class ListInterface extends AbstractInterface {
   var currentIndex(default, set):Int = 0;
   var currentScroll:Int = 0;

   public var entries:Array<ListEntry> = [];

   public var currentlySelected(get, never):ListEntry;

   function set_currentIndex(value:Int):Int {
      if (value < 0)
         value = 0;
      if (value > entries.length - 1)
         value = entries.length - 1;

      if (currentIndex == value)
         return currentIndex;

      var old = currentIndex;

      entries[value].selected();
      currentIndex = value;
      entries[old].deselected();
      return value;
   }

   function get_currentlySelected():ListEntry {
      return entries[currentIndex];
   }

   public function new() {}

   function print():Void {
      ANSI.clearScreen();
      ANSI.cursorPosition = {row: 0, column: 0};

      // Handle the scroll of the list
      if (currentIndex < currentScroll)
         currentScroll = currentIndex;
      if (currentIndex > currentScroll + ANSI.terminalSize.height - 2)
         currentScroll = currentIndex - (ANSI.terminalSize.height - 1);

      // clamp the scroll of the list
      if (currentScroll > entries.length - ANSI.terminalSize.height)
         currentScroll = entries.length - ANSI.terminalSize.height;
      if (currentScroll < 0)
         currentScroll = 0;

      // Print the list of entries
      var first = true;
      for (i in currentScroll...(currentScroll + ANSI.terminalSize.height)) {
         if (i >= entries.length)
            break;

         if (!first)
            Sys.print('\n');

         entries[i].print();
         first = false;
      }

      // if (debug) {
      //   var debugText = '${scroll}s ${height}h ${selectedIndex}i ${versionList.length}l';
      //   exec('0;${size[1] - debugText.length}f$debugText');
      // }
   }

   override function arrowUpKey():Void {
      currentIndex -= 1;
   }

   override function arrowDownKey():Void {
      currentIndex += 1;
   }

   override function arrowLeftKey():Void {
      currentlySelected.keyLeft();
   }

   override function arrowRightKey():Void {
      currentlySelected.keyRight();
   }

   override function pageUpKey():Void {
      currentIndex -= ANSI.terminalSize.height - 1;
   }

   override function pageDownKey():Void {
      currentIndex += ANSI.terminalSize.height - 1;
   }

   override function homeKey():Void {
      currentIndex = 0;
   }

   override function endKey():Void {
      currentIndex = entries.length - 1;
   }

   override function enterKey():Void {
      currentlySelected.keyEnter();
   }

   override function click(button:Int, x:Int, y:Int):Void {
      if (button != 0)
         return;

      currentIndex = currentScroll + y - 1;

      currentlySelected.click(x);
   }
}
